/* This file is part of the FaCT++ DL reasoner
Copyright (C) 2010-2015 by Dmitry Tsarkov

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef FPPTESTBASE_H
#define FPPTESTBASE_H

#include "Kernel.h"
#include "Actor.h"

//
// handy macros
//
#define C(name) TConceptExpr* name = pEM->Concept(#name)
#define I(name) TIndividualExpr* name = pEM->Individual(#name)
#define R(name)	TORoleExpr* name = pEM->ObjectRole(#name)
#define DR(name) TDRoleExpr* name = pEM->DataRole(#name)

//
// Catch all possible excepions thrown during reasoning
//

#define TryReasoning(action)			\
	do {								\
		try { action; }					\
		catch ( EFPPAssertion ass ) 	\
		{ std::cerr << ass.what() << ". Query is NOT processed\n";	\
		  exit(0); }					\
		catch ( EFPPInconsistentKB )	\
		{ std::cerr << "Ontology inconsistent. Query is NOT processed\n";	\
		  exit(0); }					\
		catch ( EFPPCantRegName crn )	\
		{ std::cerr << "Query name " << crn.getName()		\
			<< " is undefined in TBox\n"; }					\
		catch ( EFPPNonSimpleRole nsr )	\
		{ std::cerr << "WARNING: KB is incorrect: " 		\
			<< nsr.what() << ". Query is NOT processed\n";	\
		  exit(0); }					\
		catch ( EFPPCycleInRIA cir )	\
		{ std::cerr << "WARNING: KB is incorrect: " 		\
			<< cir.what() << ". Query is NOT processed\n";	\
		  exit(0); }					\
	} while (0)

/// base class for tests
class FppTestBase
{
private:	// members
		/// test name
	std::string name;

protected:	// types
	//-------------------------------------------------------------
	// Expression typedefs
	//-------------------------------------------------------------

		/// general expression
	typedef ReasoningKernel::TExpr TExpr;
		/// concept expression
	typedef ReasoningKernel::TConceptExpr TConceptExpr;
		/// individual expression
	typedef ReasoningKernel::TIndividualExpr TIndividualExpr;
		/// role expression
	typedef ReasoningKernel::TRoleExpr TRoleExpr;
		/// object role complex expression (including role chains and projections)
	typedef ReasoningKernel::TORoleComplexExpr TORoleComplexExpr;
		/// object role expression (named role and inverse of a role)
	typedef ReasoningKernel::TORoleExpr TORoleExpr;
		/// data role expression
	typedef ReasoningKernel::TDRoleExpr TDRoleExpr;
		/// data expression
	typedef ReasoningKernel::TDataExpr TDataExpr;
		/// data type expression
	typedef ReasoningKernel::TDataTypeExpr TDataTypeExpr;
		/// data value expression
	typedef ReasoningKernel::TDataValueExpr TDataValueExpr;
		/// data facet expression
	typedef const TDLFacetExpression TFacetExpr;

	//-------------------------------------------------------------
	// Actor typedefs
	//-------------------------------------------------------------
		/// simple array of entries
	typedef Actor::Array1D Array1D;
		/// 2D array of entries
	typedef Actor::Array2D Array2D;

protected:	// members
		/// reasoner
	ReasoningKernel fpp;
		/// expression manager
	TExpressionManager* pEM;
		/// actor to use
	Actor actor;
		/// 1D array of Actor's findings
	Array1D array1D;
		/// 2D array of Actor's findings
	Array2D array2D;

private:	// methods
		/// print all elements of 1D array with given split symbol
	void printArray ( std::ostream& o, const Array1D& array, char split ) const
	{
		o << "[";
		if ( !array.empty() )
		{
			Array1D::const_iterator p = array.begin(), p_end = array.end();
			o << (*p)->getName();
			while ( ++p != p_end )
				o << split << (*p)->getName();
		}
		o << "]";
	}

protected:	// methods
		/// fills in 1D array using Actor's findings
	void init1D ( void ) { actor.getFoundData(array1D); }
		/// fills in 2D array using Actor's findings
	void init2D ( void ) { actor.getFoundData(array2D); }

		/// print actor's internals as 1D array (e.g., individuals)
	void printActor1D ( std::ostream& o )
	{
		init1D();
		printArray ( o, array1D, ',' );
		o << "\n";
	}
		/// print actor's internals as 2D array (concepts, roles etc)
	void printActor2D ( std::ostream& o )
	{
		init2D();
		o << "[";
		for ( Array2D::const_iterator p = array2D.begin(), p_end = array2D.end(); p != p_end; ++p )
			printArray ( o, *p, '=' );
		o << "]\n";
	}

public:		// interface
		/// init c'tor
	FppTestBase ( const std::string& n );
		/// empty d'tor
	virtual ~FppTestBase ( void ) {}
	
		/// name of the test
	const std::string& getName ( void ) const { return name; }

		/// test itself; @return true if failed -- implemented in children
	virtual bool test ( void ) = 0;
}; // FppTestBase

#endif
