/* This file is part of the FaCT++ DL reasoner
Copyright (C) 2015 by Dmitry Tsarkov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*/

#include "FppTestBase.h"

class TestNames: public FppTestBase
{
public:		// interface
		/// init c'tor
	TestNames ( void ) : FppTestBase("names") {}
		/// empty d'tor
	virtual ~TestNames ( void ) {}

		/// test names creation and querying; @return true if failed
	virtual bool test ( void )
	{
		std::cout << "Test for " << getName() << " started\n";
		// create a concept C1
		TConceptExpr* C1 = pEM->Concept("C1");
		// create concepts C2-C4
		C(C2);
		C(C3);
		C(C4);

		// create an ordinary (object) role R1
		TORoleExpr* R1 = pEM->ObjectRole("R1");
		// create an ordinary (object) role R2
		R(R2);

		// create a data role D1
		TDRoleExpr* D1 = pEM->DataRole("D1");
		// create a data role D2
		DR(D2);

		// create an individual I1
		TIndividualExpr* I1 = pEM->Individual("I1");
		// create an individual I2
		I(I2);

		// register that entities in the KB
		// note that C2-C4, R2 and D2 are missing here
		fpp.declare(C1);
		fpp.declare(R1);
		fpp.declare(D1);
		fpp.declare(I1);
		fpp.declare(I2);
		// add axiom C1 [= E R2.C2
		TConceptExpr* Some = pEM->Exists(R2,C2);
		fpp.impliesConcepts(C1,Some);
		// now R2 and C2 are registered

		// add axiom D2 [= D1. D2 became defined
		fpp.impliesDRoles(D2,D1);

		// add axiom C2 = C3 = C4. All of them became defined
		pEM->newArgList();
		pEM->addArg(C2);
		pEM->addArg(C3);
		pEM->addArg(C4);
		fpp.equalConcepts();

		// make I1 an instance of C1
		fpp.instanceOf(I1, C1);
		// make I1 and I2 the same individual
		pEM->newArgList();
		pEM->addArg(I1);
		pEM->addArg(I2);
		fpp.processSame();

		// now classify KB
		fpp.classifyKB();

		//
		// Instance queries
		//

		// I2 is an instance of C1, as it is the same as I1
		if ( !fpp.isInstance(I2,C1) )
			return true;

		// I1 is an instance of Some = E R2.C2, as it is an instance of C1
		if ( !fpp.isInstance(I1,Some) )
			return true;

		// I1 is an instance of E R2.C4, as C4 = C2
		if ( !fpp.isInstance(I1,pEM->Exists(R2,C4)) )
			return true;

		//
		// Queries to the hierarchy
		//

		// get all concept names in the KB

		// looking for concepts
		actor.needConcepts();
		// get the indirect sub-concepts of TOP; that would be all concepts
		fpp.getSubConcepts ( pEM->Top(), /*direct=*/false, actor );
		// count them; should be 5 (C1-C4 and BOTTOM)
		std::cout << "All subs of TOP: ";
		printActor1D(std::cout);
		if ( array1D.size() != 5 )
			return true;

		// get the direct sub-concepts of TOP
		actor.needConcepts();
		fpp.getSubConcepts ( pEM->Top(), /*direct=*/true, actor );
		// count them; now should be 4, BOTTOM is not a direct sub-concept
		std::cout << "Direct subs of TOP: ";
		printActor1D(std::cout);
		if ( array1D.size() != 4 )
			return true;
		// print them in groups
		printActor2D(std::cout);
		// count the groups; should be 2: C1 and C2=C3=C4
		if ( array2D.size() != 2 )
			return true;

		// get the direct super-concepts of BOTTOM
		actor.needConcepts();
		fpp.getSupConcepts ( pEM->Bottom(), /*direct=*/true, actor );
		// count them; now should be 4, TOP is not a direct sub-concept
		std::cout << "Direct sups of BOTTOM: ";
		printActor1D(std::cout);
		if ( array1D.size() != 4 )
			return true;
		// print them in groups
		printActor2D(std::cout);
		// count the groups; should be 2: C1 and C2=C3=C4
		if ( array2D.size() != 2 )
			return true;

		// get direct object sub-properties of TOP Object role
		actor.needObjectRoles();
		fpp.getSubRoles ( pEM->ObjectRoleTop(), /*direct=*/true, actor );
		std::cout << "Direct subs of TOP_R: ";
		printActor2D(std::cout);
		// it is R1 and R2, together with inverses -R1 and -R2, so 4 of them
		if ( array2D.size() != 4 )
			return true;

		// in order to avoid inverces, use needDataRoles() in Actor (yes, it is weird)
		actor.needDataRoles();
		fpp.getSubRoles ( pEM->ObjectRoleTop(), /*direct=*/true, actor );
		std::cout << "Direct subs of TOP_R: ";
		printActor2D(std::cout);
		// now it is only R1 and R2
		if ( array2D.size() != 2 )
			return true;

		// get all direct roles (including empty role)
		actor.needDataRoles();
		fpp.getSubRoles ( pEM->ObjectRoleTop(), /*direct=*/false, actor );
		std::cout << "All subs of TOP_R: ";
		printActor2D(std::cout);
		// now it is R1, R2 and *EROLE*
		if ( array2D.size() != 3 )
			return true;

		// get all direct roles (including top role)
		actor.needDataRoles();
		fpp.getSupRoles ( pEM->ObjectRoleBottom(), /*direct=*/false, actor );
		std::cout << "All sups of BOTTOM_R: ";
		printActor2D(std::cout);
		// now it is R1, R2 and *UROLE*
		if ( array2D.size() != 3 )
			return true;

		// get direct data sub-properties of D1
		actor.needDataRoles();
		fpp.getSubRoles ( D1, /*direct=*/true, actor );
		std::cout << "Direct subs of D1: ";
		printActor2D(std::cout);
		// it is only D2
		if ( array2D.size() != 1 )
			return true;




//		std::ofstream Out("dl.res");
//		fpp.writeReasoningResult ( Out, 0 );
		return false;
	}
}; // TestNames

// create an instance of the test
TestNames test;

